package org.codehaus.mojo.exec.project5lib;

/**
 * Used for manual integrationtest of the java goal.
 *
 */
public class ExampleLibrary
{
    public ExampleLibrary() {
    }
    public boolean isAGoodDay() {
	    return true;
    }
}
