package org.codehaus.plexus.util.interpolation;

/**
 * COPIED FROM plexus-utils-1.5.15 TO SATISFY TESTS Interpolator interface. Based on existing RegexBasedInterpolator
 * interface.
 * 
 * @author cstamas
 * @deprecated Use plexus-interpolation APIs instead.
 * @version $Id$
 */
public interface Interpolator
    extends org.codehaus.plexus.interpolation.Interpolator
{
}
